package Bio::SFF;
{
  $Bio::SFF::VERSION = '0.003';
}
use strict;
use warnings FATAL => 'all';

1;

#ABSTRACT: Standard Flowgram Format for Perl



=pod

=head1 NAME

Bio::SFF - Standard Flowgram Format for Perl

=head1 VERSION

version 0.003

=head1 DESCRIPTION

This distribution offers various SFF modules. Currently only two L<readers|Bio::SFF::Reader> are provided, in the future a writer will follow.

=head1 SEE ALSO

L<Specification|http://www.ncbi.nlm.nih.gov/Traces/trace.cgi?cmd=show&f=formats&m=doc&s=format>

=head1 AUTHOR

Leon Timmermans <leont@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2011 by Leon Timmermans, Utrecht University.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut


__END__

